%% Logical Indexing Example
% From _MATLAB Techniques for Image Processing_ by Steve Eddins.

%%
% Every MATLAB user is familiar with ordinary matrix indexing
% notation.

clear
A = magic(5)

%%

A(2,3)

%%
% A(2,3) extracts the 2nd row, 3rd column of the matrix A. You
% can extract more than one row and column at the same time:

A(2:4,3:5)

%%
% When an indexing expression appears on the left-hand side of
% the equals sign, it's assignment:

A(5,5) = 100


%%
% About every 13.6 days, someone asks this question on
% comp.soft-sys.matlab:
%
%   How do I replace all the NaNs in my matrix B with 0s?
%
% This is generally followed 4.8 minutes later with this reply
% from one of the newsgroup regulars:
%
%   B(isnan(B)) = 0;
%
% For example:

B = rand(3,3);
B(2,2:3) = NaN

%%
% Replace the NaNs with zeros:
B(isnan(B)) = 0

%%
% The expression
%
%   B(isnan(B))
%
% is an example of _logical indexing_.  Logical indexing is a
% compact and expressive notation that's very useful for many
% image processing operations.
%
% Let's talk about the basic rules of logical indexing, and then
% we'll reexamine the expression B(isnan(B)).
%
% If D is a logical array, then C(D) is a logical indexing expression.
% 
% "Logical" is one of the builtin types, or classes, of MATLAB
% matrices.  Logical operators, such as == or >, produce logical
% matrices.

C = hilb(4)

%%

D = C > 0.4

%%

whos

%%
% You can see from the output of whos that the class of the
% variable D is _logical_.  The logical indexing expression
% C(D) extracts all the values of C corresponding to nonzero
% values of D and returns them as a column vector.

C(D)

%%
% Now we know enough to break down the B(isnan(B)) example to see 
% how it works.

B = rand(3,3);
B(2,2:3) = NaN;

nan_locations = isnan(B)

%%
whos nan_locations

%%
B(nan_locations)

%%
B(nan_locations) = 0

%% Connection to binary images
% Functions in the Image Processing Toolbox, as well as the
% MATLAB functions imread and imwrite, follow the convention that
% logical matrices are treated as binary (black and white)
% images.  For example, when you read a 1-bit image file using
% imread, it returns a logical matrix:

bw = imread('text.png');
imshow(bw)
whos bw

%%
% This convention, together with logical indexing, makes it very
% convenient and expressive to use binary images as pixel masks
% for extracting or operating on sets of pixels.

%% Example: Histogram of foreground pixels
% Given a gray-scale image and a binary segmentation, compute the
% histogram of just the foreground pixels in the image.
%
% Here's our original image:

I = imread('rice.png');
imshow(I)

%%
% Here's the segmentation result (computed and saved earlier), 
% represented as a binary image:

bw = imread('segmented_rice.png');
imshow(bw)

%%
% Now use the segmentation result as a logical index into the
% original image to extract the foreground pixel values.

foreground_pixels = I(bw);
whos foreground_pixels

%%
% Finally, compute the histogram of the foreground pixels.

figure
imhist(foreground_pixels)

%%
% Or use logical indexing with the complement of the segmentation
% result to compute the histogram of the background pixels.

imhist(I(~bw))

%%
% Copyright 2012-2014 The MathWorks, Inc.